/* ------------------------------------------------------------------
 * Un exemple d'implmentation du cache d'un fichier afin d'explorer
 * l'effet des algorithmes de gestion et de remplacement
 * ------------------------------------------------------------------
 * Jean-Paul Rigault
 * ------------------------------------------------------------------
 * Stratgie FIFO (First In FIrst Out)
 * ------------------------------------------------------------------
 */

#include "strategy.h"
#include "low_cache.h"
#include "cache_list.h"

/* Transforme le pointeur sur la stratgie en un pointeur sur la liste FIFO */
#define FIFO_LIST(pcache) ((struct Cache_List *)((pcache)->pstrategy))

/* ------------------------------------------------------------------
 * Fonctions externes (utlises par cache.c)
 * ------------------------------------------------------------------
 */

/* Initialisation de la stratgie 
 * ------------------------------ 
 * On alloue et initialise la liste FIFO.
*/
void *Strategy_Create(struct Cache *pcache) 
{
    /* Cration et initialisation de la liste FIFO */
    return Cache_List_Create();
}

/* Fermeture de la stratgie
 * -------------------------
 */
void Strategy_Close(struct Cache *pcache)
{
    Cache_List_Delete(FIFO_LIST(pcache));
}

/* Invalidation du cache
 * ---------------------
 * On vide la liste FIFO.
 */
void Strategy_Invalidate(struct Cache *pcache)
{
    Cache_List_Clear(FIFO_LIST(pcache));
}

/* Algorithme de remplacement de bloc
 * ---------------------------------- 

 * On prend le premier bloc invalide. S'il n'y en a plus on prend le premier
 * bloc de la liste FIFO c'est--dire le bloc le plus ancien. Comme ce bloc
 * devient le plus rcent, on le passe en fin de liste.
 */
struct Cache_Block_Header *Strategy_Replace_Block(struct Cache *pcache) 
{
    struct Cache_Block_Header *pbh;
    struct Cache_List *fifo_list = FIFO_LIST(pcache);

     /* On cherche d'abord un bloc invalide */
    if ((pbh = Get_Free_Block(pcache)) != NULL)
    {
        /* Les blocs invalides ne sont pas dans la liste ; mettons y celui-ci */
        Cache_List_Append(fifo_list, pbh);
        return pbh;
    }

    /* Sinon on prend le premier bloc de la liste FIFO et on le dplace  la
     * fin de la liste
     */
    pbh = Cache_List_Remove_First(fifo_list);
    Cache_List_Append(fifo_list, pbh);

    return pbh;    
}

/* Fonctions "rflexes" en cas de lecture et d'criture
 * ---------------------------------------------------- 
 * Ici on ne fait rien, seul l'age de la page compte, pas la frquence des accs.
 */
void Strategy_Read(struct Cache *pcache, struct Cache_Block_Header *pbh) 
{
}  
  
void Strategy_Write(struct Cache *pcache, struct Cache_Block_Header *pbh)
{
} 

/* Identification de la stratgie
 * ------------------------------
 */
char *Strategy_Name()
{
    return "FIFO";
}
